package com.ruoyi.web.controller.handle;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.enums.HandleFileType;
import com.ruoyi.common.handlefile.HandleFile;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.file.FileImportUtils;
import com.ruoyi.common.utils.file.FileToZip;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.framework.mail.service.IMailService;
import com.ruoyi.framework.web.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * @author jianglw
 * @version 1.0
 * @date 2021/1/28 17:04
 */
@RestController
@RequestMapping("/handle/encryption")
public class EncryptionController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private IMailService iMailService;
    @Value("${spring.mail.username}")
    private String from;
    @Value( ("${ruoyi.profile}") )
    private String profile;

    @Log(title = "配置文件加密", businessType = BusinessType.IMPORT)
    @PostMapping("/analysis")
    @PreAuthorize("@ss.hasPermi('handle:encryption:analysis')")
    public void upload(@RequestParam(value = "files") MultipartFile[] files,HttpServletResponse response){
        LoginUser loginUser = tokenService.getLoginUser( ServletUtils.getRequest());
        for(MultipartFile file:files){
            if (!file.isEmpty()) {
                //对文文件的全名进行截取然后在后缀名进行删选。
                int begin = Objects.requireNonNull( file.getOriginalFilename() ).indexOf(".");
                int last = file.getOriginalFilename().length();
                //获取后缀名称
                String suffix = file.getOriginalFilename().substring(begin+1, last);
                //获取处理文件方式
                HandleFile handleFile=HandleFileType.match( suffix );
                try {
                    assert handleFile != null;
                    handleFile.readFile( file ).encryptionFile().createFile( file.getOriginalFilename(),loginUser.getUsername() ,profile);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        if(FileToZip.fileToZip(profile+loginUser.getUsername(), profile+loginUser.getUsername(),
                "配置文件")){
            //发送邮件
            //FileSystemResource fileSystemResource=new FileSystemResource( new File(profile+loginUser.getUsername()+"/"+"配置文件.zip") );
            //iMailService.sendAttachmentsMail(from, loginUser.getUser().getEmail(),"配置文件","配置文件",fileSystemResource );
            //下载文件
            try {
                FileImportUtils.exportDownload(response,new File(profile+loginUser.getUsername()+"/"+"配置文件.zip"),"配置文件.zip" );
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        //删除该用户文件夹下所有临时文件
        FileUtils.deleteFiles(new File( profile+loginUser.getUsername() ));
    }


}
