package com.ruoyi.web.controller.topology;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.topology.TopologyData;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.topology.service.ITopologyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;

/**
 * @author jianglw
 * @version 1.0
 * @date 2021/3/29 10:21
 */
@RestController
@RequestMapping("/topology")
public class IndexController extends BaseController {


    @Autowired
    private ITopologyService iTopologyService;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    /**
     * 保存组态数据
     *
     * @param topologyData
     * @return AjaxResult
     */
    @PreAuthorize("@ss.hasAnyPermi('topology:params:set,topology:index:add,topology:index:edit')")
    @PostMapping("/save")
    public AjaxResult save(@RequestBody TopologyData topologyData) throws IOException {
        List<TopologyData> list =null;
        TopologyData t=null;
        if (topologyData.getId() == null) {
            iTopologyService.saveTopologyToId(topologyData );
            stringRedisTemplate.opsForValue().set(Constants.TOPOLOGY_DATA+topologyData.getId(),JSONObject.toJSONString( topologyData.getData()) );
            return AjaxResult.success( topologyData );
        }else{
            t =iTopologyService.getTopologyById( topologyData.getId() );
            if(t!=null){
                iTopologyService.updateTopologyById(topologyData);
                stringRedisTemplate.opsForValue().set( Constants.TOPOLOGY_DATA+topologyData.getId(),JSONObject.toJSONString( topologyData.getData()) );
                return AjaxResult.success( topologyData );
            }
        }
        return AjaxResult.error( "保存失败，请检查id是否存在" );

    }
    @PreAuthorize("@ss.hasPermi('topology:index:showhide')")
    @Log(title = "组态显示/隐藏", businessType = BusinessType.OTHER)
    @PostMapping("/updateTopology")
    public AjaxResult updateTopology(@RequestBody TopologyData topologyData) throws IOException {
        iTopologyService.updateTopologyById(topologyData);
        return AjaxResult.success();
    }
    @PreAuthorize( "@ss.hasAnyPermi('topology:index:list')")
    @GetMapping("/list")
    public TableDataInfo list() {
        startPage();
        TopologyData topologyData = new TopologyData();
        List<TopologyData> list = iTopologyService.getTopology( topologyData );
        for(TopologyData t:list){
            t.setData( stringRedisTemplate.opsForValue().get( Constants.TOPOLOGY_DATA+t.getId() ) );
        }
        return getDataTable(list);
    }

    @GetMapping("/getTopology/{id}")
    public AjaxResult getTopology(@PathVariable("id") Integer id) throws IOException {
        TopologyData topologyData = new TopologyData();
        topologyData.setId( id );
        TopologyData t = iTopologyService.getTopologyById( id );
        t.setData( stringRedisTemplate.opsForValue().get( Constants.TOPOLOGY_DATA+t.getId() ) );
        return AjaxResult.success( t );
    }

    @PreAuthorize("@ss.hasPermi('topology:index:delete')")
    @Log(title = "删除组态", businessType = BusinessType.DELETE)
    @DeleteMapping("/deleteTopology/{id}")
    public AjaxResult deleteTopology(@PathVariable("id") Integer id) {
        if(iTopologyService.deleteTopologyById( id )>0){
            stringRedisTemplate.delete( Constants.TOPOLOGY_DATA+id );
            return AjaxResult.success();
        }
        return AjaxResult.error( "删除失败" );
    }

    @PreAuthorize("@ss.hasPermi('topology:index:deleteTopologyHandle')")
    @Log(title = "删除组态参数配置", businessType = BusinessType.DELETE)
    @DeleteMapping("/deleteTopologyHandle/{id}")
    public AjaxResult deleteTopologyHandle(@PathVariable("id") Integer id){
        JSONObject jsonObject=JSONObject.parseObject( stringRedisTemplate.opsForValue().get( Constants.TOPOLOGY_DATA+id ) );
        jsonObject.remove( "topologyHandle" );
        stringRedisTemplate.opsForValue().set( Constants.TOPOLOGY_DATA+id,jsonObject.toJSONString() );
        return AjaxResult.success();
    }
    @GetMapping("/test")
    public void test() throws IOException {
//        JSONObject jsonObject= JSON.parseObject( "{\n" +
//                "      tag: 'numA',  //tag节点\n"   +
//                "      fontColor: 'red', //数据颜色\n" +
//                "      text: 'test', //文本内容\n" +
//                "    }" );
//        webSocketServer.sendInfo(jsonObject.toJSONString() ,"1" );
        System.out.println("他请求成功了");
    }
}
